pub mod args;
pub mod bootstrap;
pub mod git;
pub mod gitmoji;
pub mod precommit;

use std::io::Read;
use subprocess::{Exec, Redirection};
use yansi::{Color, Paint};

pub fn read_stdout(e: Exec) -> String {
    let mut p = e.stdout(Redirection::Pipe).popen().unwrap();
    let mut str = String::new();
    p.stdout.as_mut().unwrap().read_to_string(&mut str).unwrap();
    str.trim().to_string()
}

pub const TODO_REGEX: &str = r"( |#)(todo|unimplemented|refactor|wip|fix)(:|!|\n| :)";
pub const NO_COMMIT_REGEX: &str = r"(NOCOMMIT|ENSURE: )";

pub fn no_commit_amount() -> u64 {
    rg_matches(NO_COMMIT_REGEX)
}

pub fn todos_amount() -> u64 {
    rg_matches(TODO_REGEX)
}

pub fn rg_matches(regex: &str) -> u64 {
    let mut cmd = Exec::cmd("rg")
        .arg("-i")
        .arg("--json")
        .arg("--multiline")
        .arg(regex)
        .stdout(Redirection::Pipe)
        .popen()
        .unwrap();
    cmd.wait().unwrap();
    let mut str = String::new();
    cmd.stdout
        .as_mut()
        .unwrap()
        .read_to_string(&mut str)
        .unwrap();
    let last_line = str.lines().last().unwrap();
    let val: serde_json::Value = serde_json::from_str(last_line).unwrap();
    let ret = (|| {
        val.as_object()
            .unwrap()
            .get("data")?
            .as_object()?
            .get("stats")?
            .as_object()?
            .get("matches")?
            .as_number()
            .unwrap()
            .as_i64()
    })();
    ret.unwrap_or(0) as u64
}

pub fn show_rg(regex: &str) {
    Exec::cmd("rg")
        .arg("-i")
        .arg("--multiline")
        .arg(regex)
        .popen()
        .unwrap()
        .wait()
        .unwrap();
}

pub fn show_todos(count_only: bool) {
    let amount = todos_amount();
    if amount == 0 {
        println!("{}", "✨ No TODOs 🍃".paint(Color::Green).bold());
    } else {
        let emoji = match amount {
            1..=5 => "🌱",
            6..=9 => "🙂",
            10..=19 => "😅",
            20..=49 => "🔥",
            50..=99 => "🤯",
            100..=149 => "💀",
            150.. => "⚰️",
            _ => "💀",
        };

        println!(
            "{emoji} {} TODOs found.",
            if amount < 20 {
                amount.paint(Color::Yellow).bold()
            } else {
                amount.paint(Color::Red).bold()
            }
        );

        if !count_only {
            show_rg(TODO_REGEX);
        }
    }
}
